@@ -33,6 +33,7 @@
 #include <goffice/gtk/go-combo-pixmaps.h>
 #include <goffice/gtk/goffice-gtk.h>
 #include <goffice/gtk/go-font-sel.h>
+#include <goffice/gtk/go-rotation-sel.h>
 
 #include <glade/glade-xml.h>
 #include <gtk/gtkcheckbutton.h>
@@ -47,7 +48,7 @@
 #include <gdk-pixbuf/gdk-pixdata.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <math.h>
 
@@ -70,7 +71,6 @@ static GObjectClass *parent_klass;
 typedef struct {
 	GladeXML  	*gui;
 	GladeXML  	*font_gui;
-	GladeXML	*text_layout_gui;
 	GogStyle  	*style;
 	GogStyle  	*default_style;
 	GObject		*object_with_style;
@@ -892,7 +892,7 @@ font_init (StylePrefState *state, guint32 enable, GogEditor *editor, GOCmdContex
 
 	g_return_if_fail (style->font.font != NULL);
 
-	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_font_prefs", NULL, cc);
+	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_font_prefs", GETTEXT_PACKAGE, cc);
 	if (gui == NULL)
 		return;
 
@@ -924,9 +924,9 @@ font_init (StylePrefState *state, guint32 enable, GogEditor *editor, GOCmdContex
 /************************************************************************/
 
 static void
-cb_angle_changed (GtkSpinButton *spin, StylePrefState *state)
+cb_angle_changed (GORotationSel *grs, int angle, StylePrefState *state)
 {
-	gog_style_set_text_angle (state->style, gtk_spin_button_get_value (spin));
+	gog_style_set_text_angle (state->style, angle);
 	set_style (state);
 }
 
@@ -935,22 +935,15 @@ text_layout_init (StylePrefState *state, guint32 enable, GogEditor *editor, GOCm
 {
 	GogStyle *style = state->style;
 	GtkWidget *w;
-	GladeXML *gui;
 
 	if (!enable)
 		return;
 
-	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_text_layout_prefs", NULL, cc);
-	if (gui == NULL)
-		return;
-
-	state->text_layout_gui = gui;
-
-	w = glade_xml_get_widget (gui, "angle_spin");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), style->text_layout.angle);
-	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (cb_angle_changed), state);
-
-	w = glade_xml_get_widget (gui, "gog_style_text_layout_prefs");
+	w = go_rotation_sel_new ();
+	go_rotation_sel_set_rotation (GO_ROTATION_SEL (w),
+		style->text_layout.angle);
+	g_signal_connect (G_OBJECT (w), "rotation-changed",
+		G_CALLBACK (cb_angle_changed), state);
 	gog_editor_add_page (editor, w, _("Text"));
 }
 
@@ -1004,7 +997,7 @@ gog_style_populate_editor (GogStyle *style,
 
 	enable = style->interesting_fields;
 
-	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_prefs", NULL, cc);
+	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_prefs", GETTEXT_PACKAGE, cc);
 	if (gui == NULL)
 		return;
 
@@ -1250,6 +1243,19 @@ bool_prop (xmlNode *node, char const *name, gboolean *res)
 	return FALSE;
 }
 
+static gboolean
+bool_sax_prop (char const *name, char const *id, char const *val, gboolean *res)
+{
+	if (0 == strcmp (name, id)) {
+		*res = g_ascii_tolower (*val) == 't' ||
+			g_ascii_tolower (*val) == 'y' ||
+			strtol (val, NULL, 0);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 static GogFillStyle
 str_as_fill_style (char const *name)
 {
@@ -1296,34 +1302,13 @@ gog_style_line_load (xmlNode *node, GogStyleLine *line)
 	}
 	str = xmlGetProp (node, "color");
 	if (str != NULL) {
-		line->color = go_color_from_str (str);
+		go_color_from_str (str, &line->color);
 		xmlFree (str);
 	}
 	if (bool_prop (node, "auto-color", &tmp))
 		line->auto_color = tmp;
 }
 
-static void
-gog_style_line_dom_save (xmlNode *parent, xmlChar const *name,
-			 GogStyleLine const *line)
-{
-	gchar *str;
-	xmlNode *node = xmlNewDocNode (parent->doc, NULL, name, NULL);
-
-	xmlSetProp (node, (xmlChar const *) "dash", 
-		    go_line_dash_as_str (line->dash_type));
-	xmlSetProp (node, (xmlChar const *) "auto-dash",
-		    line->auto_dash ? "true" : "false");
-	str = g_strdup_printf ("%f",  line->width);
-	xmlSetProp (node, (xmlChar const *) "width", str);
-	g_free (str);
-	str = go_color_as_str (line->color);
-	xmlSetProp (node, (xmlChar const *) "color", str);
-	g_free (str);
-	xmlSetProp (node, (xmlChar const *) "auto-color",
-		    line->auto_color ? "true" : "false");
-	xmlAddChild (parent, node);
-}
 static void
 gog_style_line_sax_save (GsfXMLOut *output, char const *name,
 			 GogStyleLine const *line)
@@ -1402,8 +1387,7 @@ gog_style_gradient_load (xmlNode *node, GogStyle *style)
 	}
 	str = xmlGetProp (node, "start-color");
 	if (str != NULL) {
-		style->fill.pattern.back
-			= go_color_from_str (str);
+		go_color_from_str (str, &style->fill.pattern.back);
 		xmlFree (str);
 	}
 	str = xmlGetProp (node, "brightness");
@@ -1413,37 +1397,12 @@ gog_style_gradient_load (xmlNode *node, GogStyle *style)
 	} else {
 		str = xmlGetProp (node, "end-color");
 		if (str != NULL) {
-			style->fill.pattern.fore
-				= go_color_from_str (str);
+			go_color_from_str (str, &style->fill.pattern.fore);
 			xmlFree (str);
 		}
 	}
 }
 
-static void
-gog_style_gradient_dom_save (xmlNode *parent, GogStyle const *style)
-{
-	gchar *str;
-	xmlNode *node =  xmlNewDocNode (parent->doc, NULL, "gradient", NULL);
-
-	xmlSetProp (node, (xmlChar const *) "direction",
-		    go_gradient_dir_as_str (style->fill.gradient.dir));
-	str = go_color_as_str (style->fill.pattern.back);
-	xmlSetProp (node, (xmlChar const *) "start-color", str);
-	g_free (str);
-	if (style->fill.gradient.brightness >= 0.) {
-		str = g_strdup_printf ("%f",
-				       style->fill.gradient.brightness);
-		xmlSetProp (node, (xmlChar const *) "brightness", str);
-		g_free (str);
-	} else {
-		str = go_color_as_str (style->fill.pattern.fore);
-		xmlSetProp (node, (xmlChar const *) "end-color", str);
-		g_free (str);
-	}
-	xmlAddChild (parent, node);
-}
-
 static void
 gog_style_fill_load (xmlNode *node, GogStyle *style)
 {
@@ -1476,14 +1435,12 @@ gog_style_fill_load (xmlNode *node, GogStyle *style)
 				}
 				str = xmlGetProp (ptr, "fore");
 				if (str != NULL) {
-					style->fill.pattern.fore
-						 = go_color_from_str (str);
+					go_color_from_str (str, &style->fill.pattern.fore);
 					xmlFree (str);
 				}
 				str = xmlGetProp (ptr, "back");
 				if (str != NULL) {
-					style->fill.pattern.back
-						 = go_color_from_str (str);
+					go_color_from_str (str, &style->fill.pattern.back);
 					xmlFree (str);
 				}
 			}
@@ -1506,49 +1463,11 @@ gog_style_fill_load (xmlNode *node, GogStyle *style)
 	}
 }
 
-static void
-gog_style_fill_dom_save (xmlNode *parent, GogStyle const *style)
-{
-	gchar *str;
-	xmlNode *node = xmlNewDocNode (parent->doc, NULL, "fill", NULL);
-	xmlNode *child;
-	xmlSetProp (node, (xmlChar const *) "type",
-		    fill_style_as_str (style->fill.type));
-	xmlSetProp (node, (xmlChar const *) "is-auto",
-		    style->fill.auto_back ? "true" : "false");
-	xmlSetProp (node, (xmlChar const *) "auto-fore",
-		    style->fill.auto_fore ? "true" : "false");
-	switch (style->fill.type) {
-	case GOG_FILL_STYLE_NONE:
-		break;
-	case GOG_FILL_STYLE_PATTERN:
-		child =  xmlNewDocNode (parent->doc, NULL, "pattern", NULL);
-		xmlSetProp (child, (xmlChar const *) "type",
-			    go_pattern_as_str (style->fill.pattern.pattern));
-		str = go_color_as_str (style->fill.pattern.fore);
-		xmlSetProp (child, (xmlChar const *) "fore", str);
-		g_free (str);
-		str = go_color_as_str (style->fill.pattern.back);
-		xmlSetProp (child, (xmlChar const *) "back", str);
-		g_free (str);
-		xmlAddChild (node, child);
-		break;
-	case GOG_FILL_STYLE_GRADIENT:
-		gog_style_gradient_dom_save (node, style);
-		break;
-	case GOG_FILL_STYLE_IMAGE:
-		/* FIXME: TODO */
-		break;
-	default:
-		break;
-	}
-	xmlAddChild (parent, node);
-}
-
 static void
 gog_style_marker_load (xmlNode *node, GogStyle *style)
 {
 	char *str;
+	GOColor c;
 	GOMarker *marker = go_marker_dup (style->marker.mark);
 
 	str = xmlGetProp (node, "shape");
@@ -1562,14 +1481,16 @@ gog_style_marker_load (xmlNode *node, GogStyle *style)
 	if (str != NULL) {
 		style->marker.auto_outline_color = TRUE;
 		bool_prop (node, "auto-outline", &style->marker.auto_outline_color);
-		go_marker_set_outline_color (marker, go_color_from_str (str));
+		if (go_color_from_str (str, &c))
+			go_marker_set_outline_color (marker, c);
 		xmlFree (str);
 	}
 	str = xmlGetProp (node, "fill-color");
 	if (str != NULL) {
 		style->marker.auto_fill_color = TRUE;
 		bool_prop (node, "auto-fill", &style->marker.auto_fill_color);
-		go_marker_set_fill_color (marker, go_color_from_str (str));
+		if (go_color_from_str (str, &c))
+			go_marker_set_fill_color (marker, c);
 		xmlFree (str);
 	}
 	str = xmlGetProp (node, "size");
@@ -1580,36 +1501,6 @@ gog_style_marker_load (xmlNode *node, GogStyle *style)
 	gog_style_set_marker (style, marker);
 }
 
-static void
-gog_style_marker_dom_save (xmlNode *parent, GogStyle const *style)
-{
-	gchar *str;
-	xmlNode *node = xmlNewDocNode (parent->doc, NULL, "marker", NULL);
-
-	xmlSetProp (node, (xmlChar const *) "auto-shape",
-		style->marker.auto_shape ? "true" : "false");
-	xmlSetProp (node, (xmlChar const *) "shape",
-		go_marker_shape_as_str (go_marker_get_shape (style->marker.mark)));
-
-	xmlSetProp (node, (xmlChar const *) "auto-outline",
-		style->marker.auto_outline_color ? "true" : "false");
-	str = go_color_as_str (go_marker_get_outline_color (style->marker.mark));
-	xmlSetProp (node, (xmlChar const *) "outline-color", str);
-	g_free (str);
-
-	xmlSetProp (node, (xmlChar const *) "auto-fill",
-		style->marker.auto_fill_color ? "true" : "false");
-	str = go_color_as_str (go_marker_get_fill_color (style->marker.mark));
-	xmlSetProp (node, (xmlChar const *) "fill-color", str);
-	g_free (str);
-
-	str = g_strdup_printf ("%d", go_marker_get_size (style->marker.mark));
-	xmlSetProp (node, (xmlChar const *) "size", str);
-	g_free (str);
-
-	xmlAddChild (parent, node);
-}
-
 static void
 gog_style_marker_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
@@ -1637,7 +1528,7 @@ gog_style_font_load (xmlNode *node, GogStyle *style)
 
 	str = xmlGetProp (node, "color");
 	if (str != NULL) {
-		style->font.color = go_color_from_str (str);
+		go_color_from_str (str, &style->font.color);
 		xmlFree (str);
 	}
 	str = xmlGetProp (node, "font");
@@ -1653,23 +1544,6 @@ gog_style_font_load (xmlNode *node, GogStyle *style)
 		style->font.auto_scale = tmp;
 }
 
-static void
-gog_style_font_dom_save (xmlNode *parent, GogStyle const *style)
-{
-	gchar *str;
-	xmlNode *node = xmlNewDocNode (parent->doc, NULL, "font", NULL);
-
-	str = go_color_as_str (style->font.color);
-	xmlSetProp (node, (xmlChar const *) "color", str);
-	g_free (str);
-	str = go_font_as_str (style->font.font);
-	xmlSetProp (node, (xmlChar const *) "font", str);
-	g_free (str);
-	xmlSetProp (node, (xmlChar const *) "auto-scale",
-		    style->font.auto_scale ? "true" : "false");
-
-	xmlAddChild (parent, node);
-}
 static void
 gog_style_font_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
@@ -1695,19 +1569,6 @@ gog_style_text_layout_load (xmlNode *node, GogStyle *style)
 	}
 }
 
-static void
-gog_style_text_layout_dom_save (xmlNode *parent, GogStyle const *style)
-{
-	gchar *str;
-	xmlNode *node = xmlNewDocNode (parent->doc, NULL, "text_layout", NULL);
-
-	str = g_strdup_printf ("%g", style->text_layout.angle);
-	xmlSetProp (node, (xmlChar const *) "angle", str);
-	g_free (str);
-
-	xmlAddChild (parent, node);
-}
-
 static void
 gog_style_text_layout_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
@@ -1743,25 +1604,120 @@ gog_style_persist_dom_load (GogPersist *gp, xmlNode *node)
 }
 
 static void
-gog_style_persist_dom_save (GogPersist const *gp, xmlNode *parent)
+gog_style_sax_load_line (GsfXMLIn *xin, xmlChar const **attrs)
 {
-	GogStyle const *style = GOG_STYLE (gp);
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	GogStyleLine *line = xin->node->user_data.v_int ? &style->outline : &style->line;
 
-	xmlSetProp (parent, (xmlChar const *) "type",
-		G_OBJECT_TYPE_NAME (style));
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (0 == strcmp (attrs[0], "dash"))
+			line->dash_type = go_line_dash_from_str (attrs[1]);
+		else if (bool_sax_prop ("auto-dash", attrs[0], attrs[1], &line->auto_dash))
+			;
+		else if (0 == strcmp (attrs[0], "width")) {
+			line->width = g_strtod (attrs[1], NULL);
+			/* For compatibility with older graphs, when dash_type
+			 * didn't exist */
+			if (line->width < 0.) {
+				line->width = 0.;
+				line->dash_type = GO_LINE_NONE;
+			}
+		} else if (0 == strcmp (attrs[0], "color"))
+			go_color_from_str (attrs[1], &line->color);
+		else if (bool_sax_prop ("auto-color", attrs[0], attrs[1], &line->auto_color))
+			;
+}
+static void
+gog_style_sax_load_fill_pattern (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	g_return_if_fail (style->fill.type == GOG_FILL_STYLE_PATTERN);
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (0 == strcmp (attrs[0], "type"))
+			style->fill.pattern.pattern = go_pattern_from_str (attrs[1]);
+		else if (0 == strcmp (attrs[0], "fore"))
+			go_color_from_str (attrs[1], &style->fill.pattern.fore);
+		else if (0 == strcmp (attrs[0], "back"))
+			go_color_from_str (attrs[1], &style->fill.pattern.back);
+}
 
-	if (style->interesting_fields & GOG_STYLE_OUTLINE)
-		gog_style_line_dom_save (parent, "outline", &style->outline);
-	if (style->interesting_fields & GOG_STYLE_LINE)
-		gog_style_line_dom_save (parent, "line", &style->line);
-	if (style->interesting_fields & GOG_STYLE_FILL)
-		gog_style_fill_dom_save (parent, style);
-	if (style->interesting_fields & GOG_STYLE_MARKER)
-		gog_style_marker_dom_save (parent, style);
-	if (style->interesting_fields & GOG_STYLE_FONT)
-		gog_style_font_dom_save (parent, style);
-	if (style->interesting_fields & GOG_STYLE_TEXT_LAYOUT)
-		gog_style_text_layout_dom_save (parent, style);
+static void
+gog_style_sax_load_fill_gradient (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	g_return_if_fail (style->fill.type == GOG_FILL_STYLE_GRADIENT);
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (0 == strcmp (attrs[0], "direction"))
+			style->fill.gradient.dir = go_gradient_dir_from_str (attrs[1]);
+		else if (0 == strcmp (attrs[0], "start-color"))
+			go_color_from_str (attrs[1], &style->fill.pattern.back);
+		else if (0 == strcmp (attrs[0], "end-color"))
+			go_color_from_str (attrs[1], &style->fill.pattern.fore);
+		else if (0 == strcmp (attrs[0], "brightness"))
+			gog_style_set_fill_brightness (style, g_strtod (attrs[1], NULL));
+}
+
+static void
+gog_style_sax_load_fill (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (0 == strcmp (attrs[0], "type"))
+			style->fill.type = str_as_fill_style (attrs[1]);
+		else if (bool_sax_prop ("is-auto", attrs[0], attrs[1], &style->fill.auto_back))
+			;
+		else if (bool_sax_prop ("auto-fore", attrs[0], attrs[1], &style->fill.auto_fore))
+			;
+}
+static void
+gog_style_sax_load_marker (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	GOMarker *marker = go_marker_dup (style->marker.mark);
+	GOColor c;
+
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (bool_sax_prop ("auto-shape", attrs[0], attrs[1], &style->marker.auto_shape))
+			;
+		else if (0 == strcmp (attrs[0], "shape"))
+			go_marker_set_shape (marker, go_marker_shape_from_str (attrs[1]));
+		else if (bool_sax_prop ("auto-outline", attrs[0], attrs[1], &style->marker.auto_outline_color))
+			;
+		else if (0 == strcmp (attrs[0], "outline-color")) {
+			if (go_color_from_str (attrs[1], &c))
+				go_marker_set_outline_color (marker, c);
+		} else if (bool_sax_prop ("auto-fill", attrs[0], attrs[1], &style->marker.auto_fill_color))
+			;
+		else if (0 == strcmp (attrs[0], "fill-color")) {
+			if (go_color_from_str (attrs[1], &c))
+				go_marker_set_fill_color (marker, c);
+		} else if (0 == strcmp (attrs[0], "size"))
+			go_marker_set_size (marker, g_strtod (attrs[1], NULL));
+
+	gog_style_set_marker (style, marker);
+}
+
+static void
+gog_style_sax_load_font (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (0 == strcmp (attrs[0], "color"))
+			go_color_from_str (attrs[1], &style->font.color);
+		else if (0 == strcmp (attrs[0], "font")) {
+			PangoFontDescription *desc = pango_font_description_from_string (attrs[1]);
+			if (desc != NULL)
+				gog_style_set_font_desc (style, desc);
+		} else if (bool_sax_prop ("auto-scale", attrs[0], attrs[1], &style->font.auto_scale))
+			;
+}
+static void
+gog_style_sax_load_text_layout (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GogStyle *style = GOG_STYLE (gog_xml_read_state_get_obj (xin));
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (0 == strcmp (attrs[0], "angle"))
+			gog_style_set_text_angle (style, g_strtod (attrs[1], NULL));
 }
 
 static void
@@ -1786,11 +1742,32 @@ gog_style_persist_sax_save (GogPersist const *gp, GsfXMLOut *output)
 		gog_style_text_layout_sax_save (output, style);
 }
 
+static void
+gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs)
+{
+	static GsfXMLInNode const dtd[] = {
+	  GSF_XML_IN_NODE (STYLE, STYLE, -1, "GogObject", FALSE, NULL, NULL),
+	    GSF_XML_IN_NODE_FULL (STYLE, STYLE_LINE,	-1, "line", GSF_XML_NO_CONTENT, FALSE, FALSE, &gog_style_sax_load_line, NULL, 0),
+	    GSF_XML_IN_NODE_FULL (STYLE, STYLE_OUTLINE,	-1, "outline", GSF_XML_NO_CONTENT, FALSE, FALSE, &gog_style_sax_load_line, NULL, 1),
+	    GSF_XML_IN_NODE (STYLE, STYLE_FILL,		-1, "fill", GSF_XML_NO_CONTENT, &gog_style_sax_load_fill, NULL),
+	      GSF_XML_IN_NODE (STYLE_FILL, FILL_PATTERN,  -1, "pattern", GSF_XML_NO_CONTENT, &gog_style_sax_load_fill_pattern, NULL),
+	      GSF_XML_IN_NODE (STYLE_FILL, FILL_GRADIENT, -1, "gradient", GSF_XML_NO_CONTENT, &gog_style_sax_load_fill_gradient, NULL),
+	    GSF_XML_IN_NODE (STYLE, STYLE_MARKER,	-1, "marker", GSF_XML_NO_CONTENT, &gog_style_sax_load_marker, NULL),
+	    GSF_XML_IN_NODE (STYLE, STYLE_FONT,		-1, "font", GSF_XML_NO_CONTENT, &gog_style_sax_load_font, NULL),
+	    GSF_XML_IN_NODE (STYLE, STYLE_ALIGNMENT,	-1, "text_layout", GSF_XML_NO_CONTENT, &gog_style_sax_load_text_layout, NULL),
+	  GSF_XML_IN_NODE_END
+	};
+	static GsfXMLInDoc *doc = NULL;
+	if (NULL == doc)
+		doc = gsf_xml_in_doc_new (dtd, NULL);
+	gsf_xml_in_push_state (xin, doc, NULL, NULL, attrs);
+}
+
 static void
 gog_style_persist_init (GogPersistClass *iface)
 {
 	iface->dom_load = gog_style_persist_dom_load;
-	iface->dom_save = gog_style_persist_dom_save;
+	iface->prep_sax = gog_style_persist_prep_sax;
 	iface->sax_save = gog_style_persist_sax_save;
 }
 
